@@ -4,13 +4,13 @@ from qtpy import QtCore
 from qtpy.QtCore import Signal
 
 from .. import core
-from .. import git
 from .. import hidpi
 from .. import utils
 from ..cmd import Command
 
 
 ABBREV = 'core.abbrev'
+ASPELL_ENABLED = 'cola.aspell.enabled'
 AUTOCOMPLETE_PATHS = 'cola.autocompletepaths'
 AUTODETECT_PROXY = 'cola.autodetectproxy'
 AUTOTEMPLATE = 'cola.autoloadcommittemplate'
@@ -29,6 +29,7 @@ EDITOR = 'gui.editor'
 ENABLE_GRAVATAR = 'cola.gravatar'
 EXPANDTAB = 'cola.expandtab'
 FONTDIFF = 'cola.fontdiff'
+FONTSIZE = 'cola.fontsize'
 HIDPI = 'cola.hidpi'
 HISTORY_BROWSER = 'gui.historybrowser'
 HTTP_PROXY = 'http.proxy'
@@ -108,8 +109,10 @@ class Defaults:
 
     # These should match Git's defaults for git-defined values.
     abbrev = 12
+    aspell_enabled = False
     autotemplate = False
     autodetect_proxy = True
+    background_editor = ''
     blame_viewer = 'git gui blame'
     block_cursor = True
     bold_headers = False
@@ -161,7 +164,7 @@ def abbrev(context):
     default = Defaults.abbrev
     value = context.cfg.get(ABBREV, default=default)
     if value == 'no':
-        result = git.OID_LENGTH_SHA256
+        result = context.model.oid_len
     else:
         try:
             result = max(4, int(value))
@@ -170,6 +173,11 @@ def abbrev(context):
     return result
 
 
+def aspell_enabled(context):
+    """Return True when aspell should be used as the spelling dictionary source"""
+    return context.cfg.get(ASPELL_ENABLED, default=Defaults.aspell_enabled)
+
+
 def autodetect_proxy(context):
     """Return True when proxy settings should be automatically configured"""
     return context.cfg.get(AUTODETECT_PROXY, default=Defaults.autodetect_proxy)
@@ -273,9 +281,7 @@ def default_history_browser():
         # because we don't have to guess paths.
         git_cola = sys.argv[0].replace('\\', '/')
         python = sys.executable.replace('\\', '/')
-        cwd = core.getcwd().replace('\\', '/')
-        argv = [python, git_cola, 'dag', '--repo', cwd]
-        argv = core.prep_for_subprocess(argv)
+        argv = core.prep_for_subprocess([python, git_cola, 'dag'])
         default = core.list2cmdline(argv)
     else:
         # The `gitk` script can be launched as-is on unix
