@@ -554,7 +554,6 @@ class SaveBlob(cmds.ContextCommand):
         self.browse_model = model
 
     def do(self):
-        git = self.context.git
         model = self.browse_model
         if model.ref == dag.WORKTREE:
             try:
@@ -579,7 +578,7 @@ class SaveBlob(cmds.ContextCommand):
                 model_ref = model.ref
             ref = f'{model_ref}:{model.relpath}'
             with core.xopen(model.filename, 'wb') as fp:
-                status, output, err = git.cat_file('blob', ref, _stdout=fp)
+                status, output, err = self.context.git.cat_file('blob', ref, _stdout=fp)
 
             out = '# git cat-file blob {} >{}\n{}'.format(
                 shlex.quote(ref),
@@ -816,8 +815,9 @@ class GitTreeModel(GitFileTreeModel):
 
     def _initialize(self):
         """Iterate over git-ls-tree and create GitTreeItems."""
-        git = self.context.git
-        status, out, err = git.ls_tree('--full-tree', '-r', '-t', '-z', self.ref)
+        status, out, err = self.context.git.ls_tree(
+            '--full-tree', '-r', '-t', '-z', self.ref
+        )
         if status != 0:
             Interaction.log_status(status, out, err)
             return
@@ -825,12 +825,13 @@ class GitTreeModel(GitFileTreeModel):
         if not out:
             return
 
+        oid_len = self.context.model.oid_len
         for line in out[:-1].split('\0'):
             # .....6 ...4 ......................................40
             # 040000 tree c127cde9a0c644a3a8fef449a244f47d5272dfa6	relative
             # 100644 blob 139e42bf4acaa4927ec9be1ec55a252b97d3f1e2	relative/path
             objtype = line[7]
-            relpath = line[6 + 1 + 4 + 1 + 40 + 1 :]
+            relpath = line[6 + 1 + 4 + 1 + oid_len + 1 :]
             if objtype == 't':
                 parent = self.dir_entries[utils.dirname(relpath)]
                 self.add_directory(parent, relpath)
